import ctypes
from typing import List, Optional
from .__library__ import lib, get_error


class CrashReport():
    """
    Class containing log information about the crash that occurred
    """
    def __del__(self):
        if self._ptr:
            lib.tof_crash_report_delete(self._ptr)
            self._ptr = ctypes.c_void_p()


    def __init__(self, serial: Optional[str] = None, description: Optional[str] = None, ptr: Optional[ctypes.c_void_p] = None):
        """
        Construct new crash_report class
        ----------

        Generate crash report on a camera

        Parameters
        ----------
        serial : str
                Camera to generate the crash report on
        description : str
                Description of what happened
        """
        self._ptr = ctypes.c_void_p()

        if ptr != None:
            self._ptr = ptr
        elif serial != None and description != None:
            self._ptr = lib.tof_crash_report_new(serial.encode("utf-8"), description.encode("utf-8"), get_error())
        else:
            raise RuntimeError("Can't construct empty class")

    def getDescription(self) -> str:
        """
        Get the description

        Returns
        ----------
        str
            Description
        """
        return lib.tof_crash_report_get_description(self._ptr, get_error()).decode("utf-8")


    def setDescription(self, description: str) -> None:
        """
        Set the description

        Parameters
        ----------
        description : str
            Description
        """
        return lib.tof_crash_report_set_description(self._ptr, description.encode("utf-8"), get_error())


    def getReport(self) -> str:
        """
        Get the complete report

        Returns
        ----------
        str
            String of the entire report
        """
        return lib.tof_crash_report_get_report(self._ptr, get_error()).decode("utf-8")


    def save(self, file_name: str) -> None:
        """
        Save report to disk

        Parameters
        ----------
        file_name : str
            File to save the report to
        """
        return lib.tof_crash_report_save(self._ptr, file_name.encode("utf-8"), get_error())


def getVersion() -> str:
    """
    Get the version of the tof library

    Returns
    ----------
    str
        Library version
    """
    return lib.tof_get_version(get_error()).decode("utf-8")


def logToConsole() -> None:
    """
    Log info/warning/error occuring in the tof library to stdout
    """
    return lib.tof_log_to_console(get_error())


def logToFile(file_location: str) -> None:
    """
    Log info/warning/error occuring in the tof library

    Parameters
    ----------
    file_location : str
        Location where to store the log file
    """
    return lib.tof_log_to_file(file_location.encode("utf-8"), get_error())


def logDrop() -> None:
    """
    Stop logging
    """
    return lib.tof_log_drop(get_error())


